/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MTARRAY_
#define _MTARRAY_

// Use for debugging, adds much processing
//#define MAGIC_SUM
#define MARRAY_MAGIC	0x1acf0001

struct _madisk
{
	int	magic;
	int	max;
	int 	num;
	int	cb;
};

/** A template array class */
template <class TYPE> class MTArray
{
	#define GROW(x)	if(x >=  m_elementMax) { if(!grow(x)) throw((void *)NULL); }
	protected:
	Boolean m_fClear;
	Boolean	m_fCloned;
	Boolean	m_fstunted;
	int 	m_elementMax;
	TYPE *	m_array;

	/** Reallocate to newmax */
	Boolean setmax(int newmax)
	{
		TYPE *p;

		if(newmax < m_elementMax)
			return TRUE;

		if(m_fstunted != TRUE)
		{
			unsigned int cb = newmax * sizeof(TYPE);
		
			// will do initial malloc if m_table==NULL
			p = (TYPE *)realloc((TYPE *)m_array, cb);

			if(p)
			{
				if(m_fClear)
				{
					int cb = (newmax - m_elementMax)*sizeof(TYPE);
					memset((void *)&p[m_elementMax],0,cb); 
				}
				m_array = p;
				m_elementMax = newmax;
				return TRUE;
			}
			else
			{
				printf("realloc failed\n");
				p = (TYPE *) malloc(cb);
				assert(p);
				memcpy(p, m_array, sizeof(TYPE) * m_Count);
				free((void*)m_array);

				if(m_fClear)
				{
					int cb = (newmax - m_elementMax)*sizeof(TYPE);
					memset((void *)&p[m_elementMax],0,cb); 
				}
				m_array = p;
				m_elementMax = newmax;
				return TRUE;
			}
		}
		return FALSE;
	}
	
	/** Grow to allow n items in array */
	Boolean grow(int n)
	{
		int newmax;

		if(m_elementMax > n)
			return TRUE;

		if(m_fstunted == TRUE)
			return FALSE;

		newmax = m_elementMax * 2;

		if(newmax < n)
			newmax = n+1;

		return setmax(newmax);
	}

	public:
	MTArray(int start=4)
	{
		m_fClear=TRUE;
		m_fCloned = FALSE;
		m_elementMax=0;
		m_Count=0;
		m_array = NULL;
		m_fstunted = FALSE;
		setmax(start);
	}
	MTArray(MRandomFile *mf)
	{
		m_fCloned = FALSE;
		m_elementMax=0;
		m_Count=0;
		m_array = NULL;
		m_fstunted = FALSE;

		if(mf)
			load(mf);
	}
	MTArray(TYPE *t, int count)
	{
		m_fCloned = FALSE;
		m_elementMax=count;
		m_Count=count;
		m_array = t;
		m_fstunted = TRUE;
	}
	~MTArray()
	{
		if(m_array)
			free(m_array);
	}
	/** Erase all information */
	void clear(void)
	{
		memset(m_array,0, sizeof(TYPE) * m_elementMax);
	}
#if 0
	/** search array for t */
	int find(TYPE t)
	{
		for(int i=0; i < m_Count; i++)
		{
			if(t == m_array[i])
				return i;
		}
		return -1;
	}
#endif
	/** classic [] operator */
	TYPE & operator [] (int ndx)
	{
		if(ndx < m_Count)
			return m_array[ndx];
		else
		{
			GROW(ndx);
			m_Count = MAX(m_Count, ndx+1);
			return m_array[ndx];
		}
	}

	TYPE *GetNextPtr(int *pndx)
	{
		GROW(m_Count);
		*pndx = m_Count;
		return &m_array[m_Count++];
	}
	
	/** Get a specific item */
	TYPE & get (int ndx)
	{
		if(ndx < m_Count)
			return m_array[ndx];
		else
		{
			GROW(ndx);
			m_Count = MAX(m_Count, ndx+1);
			return m_array[ndx];
		}
	}
	TYPE * GetPtr(int ndx)
	{
		return ((ndx < m_Count) && (ndx > -1)) ? &m_array[ndx] : NULL;
	}
	
	/** add t into next slot, grow if needed */
	TYPE & Add(TYPE & t)
	{
		GROW(m_Count);
		return m_array[m_Count++]=t;
	}
	
	/** add t into next slot, grow if needed, return a pointer! */
	TYPE *AddPtr(TYPE & t)
	{
		GROW(m_Count);
		int ndx = m_Count++;
		m_array[ndx]=t;
		return &m_array[ndx];
	}

	/** add t and return it's index */
	int AddNdx(TYPE & t)
	{
		GROW(m_Count);
		int ndx = m_Count;
		m_array[m_Count++]=t;
		return ndx;
	}
	/** add t and return it's index */
	TYPE *AddNdxPtr(TYPE & t, int *pndx)
	{
		GROW(m_Count);
		int ndx = m_Count++;
		m_array[ndx]=t;
		*pndx = ndx;
		return &m_array[ndx];
	}
	/** set t into specific slot */
	TYPE & Set(TYPE & t, int ndx)
	{
		GROW(ndx);
		m_Count = MAX(m_Count, ndx+1);
		return m_array[ndx] = t;
	}

	/** return a pointer to the array base */
	TYPE *pointer(void)
	{
		return	m_array;
	}
	/** return the max size of the array so far */
	int sizeof_max_array()
	{
		return (sizeof( TYPE ) * m_elementMax); 
	}
	/** return memory size of used slots */
	int sizeof_array()
	{
		return (sizeof( TYPE ) * m_Count);
	}
	/** return number of objects in array */
	int count()
	{
		return m_Count;
	}
	/** Set the count of items in array */
	Boolean setcount(int newcount)
	{
		if(setmax(newcount))
		{
			m_Count = newcount;
			return TRUE;
		}
		else
			return FALSE;
	}
	/** return size of file image of this array */
	int sizeof_save()
	{
		return (sizeof( TYPE ) * m_Count)+sizeof(struct _madisk);
	}
#if 0
	/** sum array. WARNING: assumes integers, could use type, I guess. */
	int arrsum(int count)
	{
		int i;
		int sum;
		for(i=0,sum=0; i < count; i++) 
			sum+= ((int)m_array[i]);
		return sum;
	}
#endif


	/** load array from a file */
	int load(MFile *mf)
	{
		struct _madisk ma;

		if(mf->read(&ma, sizeof(ma)) != sizeof(ma))
			return FALSE;

		assert(m_Count == 0);

#ifndef MAGIC_SUM
		assert(ma.magic == MARRAY_MAGIC);
#endif
		assert(ma.num <= ma.max);

		if(!setmax(ma.max))
			return 0;

		int cb = ma.num * sizeof( TYPE );

		assert(ma.cb == cb);

		if(mf->read(&m_array[m_Count], cb)== cb)
		{
#ifdef MAGIC_SUM
			assert(arrsum(ma.num) == ma.magic);
#endif
			m_Count+=ma.num;
			return cb + sizeof(ma);
		}
		return 0;
	}

	/** Save array to a file */
	int save(MFile *mf)
	{
		struct _madisk ma;
#ifdef MAGIC_SUM
		ma.magic = arrsum(m_Count);
#else
		ma.magic = MARRAY_MAGIC;
#endif
		ma.max = m_elementMax;
		ma.num = m_Count;
		ma.cb = sizeof( TYPE ) * m_Count;

		if(mf->write(&ma,sizeof(ma)) != sizeof(ma))
			return 0;

		if(mf->write(m_array, ma.cb) != ma.cb)
			return 0;

		return ma.cb + sizeof(ma);
	}

	/** shrink the array to used objects */
	void shrink()
	{
		m_elementMax = m_Count;
	}
	/** truncate the array at ndx. (0 clears it) */
	void trunk(int ndx)
	{
		m_Count = MIN(m_Count, ndx);
	}

	/** Creates a readonly clone of an array. Assumes original does not shrink! */
	void clone( MTArray & mta)
	{
		free(m_array);
		m_fCloned = TRUE;
		m_fstunted = TRUE;
		m_elementMax = mta.m_elementMax;
		m_array = mta.m_array;
		m_Count = mta.m_Count;
		
	}
	int 	m_Count;
};

typedef class _inttuple
{
public:
	int 	a;
	int	b;
	_inttuple & operator = (_inttuple & x)
	{
		a = x.a;
		b = x.b;
		return x;
	}
	int operator - (const _inttuple & x)
	{
		return a - x.a;
	}
}ITUPLE;

/** An array of integer tuples */
class MITuple : public MTArray<ITUPLE>, protected MTSort<ITUPLE>
{
	protected:
	static int intsorta(void *p1, void *p2);
	static int intsortb(void *p1, void *p2);
	static int intsortad(void *p1, void *p2);
	static int intsortbd(void *p1, void *p2);

	public:
	void Sorta(Boolean fDecending=FALSE);
	void Sortb(Boolean fDecending=FALSE);

};
#endif
