/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef __AVL__
/**
A generic AVL class for searchable nodes
*/
class MAVL : public MNode, public MNodeHolder
{
	protected:
	static Boolean defwalker(MNode *mn, void *param1, void *param2);

	unsigned int	m_state;
	MNode *		m_top;
	MNode *		m_last;

	/** Internal: Insert a node into the tree */
	MNode *	Insert(MNode *top, MNode *insert);

	/** Internal: Insert a node (known to be a string) into the tree */
	MNode *MAVL::InsertStr(MNode *cnT, MNode *insert);
	/** Internal: for traversing the tree */
	Boolean	TreeWalkUp(MNode *, EnumFn fn, void *p1, void *p2);
	/** Internal: for traversing the tree */
	Boolean	TreeWalkDn(MNode *, EnumFn fn, void *p1, void *p2);

	/** Magic AVL management */
        static MNode * 	SingleRotateLeft(MNode * left);
	/** Magic AVL management */
	static MNode * 	SingleRotateRight(MNode * right);
	/** Magic AVL management */
	static MNode * 	DoubleRotateLeft(MNode * right);
	/** Magic AVL management */
	static MNode * 	DoubleRotateRight(MNode * left);
	/** Magic AVL management */

	public:
			MAVL();
	virtual		~MAVL();

	void 		clear(void);

	/** Walk the tree */
	void 		Enum(EnumFn fn, void *param1, void *param2, Boolean ascending=TRUE);

	// MNodeHolder
	/** find a node in the tree */
	MNode *		Find(NKey key);
	/** Insert a node in to the tree */
	Boolean		Insert(MNode *insert);

	/** Insert a known string node in to the tree */
	Boolean		InsertStr(MNode *insert);
	/** find a string node with a string
	This is a hack for speed!!!
	*/
	inline MNode *FindStr(char *key)
	{
		if(m_last && !strcmp(key, (char *)m_last->m_key))
			return m_last;
		else
		{
			MNode *node = m_top;
			while(node)
			{
				int res = key[0] - ((char *)node->m_key)[0];
				res = (res) ? res : strcmp(key, (char *)node->m_key) ;
				if(res)
					node = (res < 0) ?  (MNode *) node->m_prev : (MNode *) node->m_next;	
				else
				{
					m_last = node;
					return node;
				}
			}
		}
		return NULL;
	}
	inline MNode *MAVL::FindInt(int key)
	{
		if(m_last && key == (int)m_last->m_key)
			return m_last;
		else
		{
			int res;
			MNode *node = m_top;
			while(node)
			{
				res = key - (int)node->m_key;
				if(!res)
				{
					m_last = node;
					return node;
				}
				node = (res < 0) ?  
					(MNode *) node->m_prev : 
					(MNode *) node->m_next;
			}
		}
		return NULL;
	}
};

class MAVLD : public MAVL
{
	public:
	~MAVLD() { DeleteAll(NULL,NULL,NULL); }
};

#endif
