/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MEMHEAP_
#define _MEMHEAP_

/** A file persistent memory heap. More efficient than a blockfile, intended for a 
large number of strings.
Note: a memheap handle is not compatible with block file handles.
*/
class MemHeap
{
	#define SHDR_MAGIC 1010001

	typedef struct _memheap_hdr
	{
		unsigned int magic;
		unsigned int cbarray;
		unsigned int cbheap;
		unsigned int reserved;
	}SHDR;

	typedef struct _headndx
	{
		unsigned int offset;
		unsigned int prop;
	}SNDX;

	protected:
	MTArray	<SNDX>		m_offsets;
	char *			m_memheap;
	unsigned int		m_end;
	unsigned int		m_size;
	unsigned int		m_step;
	Boolean			m_fCloned;

	Boolean grow(int);

	inline Boolean check_size(int cb)
	{ 
		return ((m_end + cb) < m_size) ? TRUE : grow(cb); 
	}
	

	public:
	MemHeap(int size=1024);
	MemHeap(MemHeap *mh);
	virtual ~MemHeap();
	Boolean save(MFile *f);
	Boolean load(MFile *f);
	int	adds(char *str);
	int	adds(char *str, int ndx);


	/** Get a property of a handle */
	unsigned int getprop(int handle);
	/** Set a property of a handle */
	void setprop(int handle, unsigned int prop);
	/** Clone and copy */
	void clone( MemHeap & mh);

	char *strdup(char *string);
	void *malloc(int cb);
	void *memdup(int cb, void *mem);
	void clear();

	inline void *MemHeap::operator [] (int handle)
	{ 
		return (handle == -1) ? NULL : (void *)&m_memheap[ m_offsets [ handle ].offset ]; 
	}
	inline int addm(int cb, void *mem)
	{
		if(check_size(cb))
		{
			SNDX sndx;
			sndx.prop = (unsigned int) cb;
			sndx.offset = m_end;
			m_end 	+= cb;
			if(mem)
			{
				char *p = &m_memheap[sndx.offset];
				memcpy(p, mem, cb);
			}
			return m_offsets.AddNdx( sndx );
		}
		return -1;
	}
	inline int addm(int cb, void *mem, int ndx)
	{
		if(check_size(cb))
		{
			SNDX sndx;
			sndx.prop = (unsigned int) cb;
			sndx.offset = m_end;
			m_end 	+= cb;
			if(mem)
			{
				char *p = &m_memheap[sndx.offset];
				memcpy(p, mem, cb);
			}
			m_offsets.Set( sndx, ndx );
			return ndx;
		}
		return -1;
	}
};

#endif
