/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MHIST_
#define _MHIST_

/** The base class of the MTHistogram template 
Uses a nodes Meta and Count values. */
class MHistogramBase
{
	protected:
	MNode **		m_table;
	MAVL			m_index;
	int			m_entries;
	int			m_maxent;

	Boolean	tableGrow(int n);
	MNode *	indexAdd(MNode *mn);
	MNode * indexFind(NKey key);

	public:
	MHistogramBase(int init=64);
	~MHistogramBase();
	MNode **calc(int *ce);
	void clear(void);
};

/** Template class for histograms */
template <class KEY> class MTHistogram : public MHistogramBase
{	
	public:		
	MTHistogram(int init=63) : MHistogramBase(init)
	{ }
	MNode *Add(KEY key)					
	{											
		MNode *mn = indexFind((NKey)key);
		if(!mn)
		{
			mn = new (key) MNode();
			return indexAdd(mn);		
		}
		else
			mn->CountInc();						
		return mn;
	}
	KEY operator [] (int ndx)
	{
		return (ndx >0 && ndx < m_entries) ?
			(KEY) m_table[ndx]->KeyGet() : (KEY) NULL;
	}
};

class MIntHistogram : public MHistogramBase
{
	public:
	MIntHistogram(int init=63) : MHistogramBase(init)
	{ }

	MNode *Add(int key)
	{
		MNode * mn = indexFind((NKey)key);

		if(!mn)
		{
			mn = new MIntNode(key);
			return indexAdd(mn);		
		}
		else
			mn->CountInc();
		return mn;
	}
	int operator [] (int ndx)
	{
		return (ndx >0 && ndx < m_entries) ?
			(int) m_table[ndx]->KeyGet() : (int) NULL;
	}
	
};

#endif
