/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 

#ifndef _MIPAIR_
#define _MIPAIR_

typedef unsigned int MPLKEY;
typedef unsigned int MPLVAL;

/** An awkward sorted list sort of thing. */
class MIPairList : public MIntNode
{
	struct IPAIR
	{
		MPLKEY	key;
		MPLVAL	value;
	};

	protected:
	unsigned int m_lastKey;
	int	m_lastNdx;
	int	m_scan;
	int 	m_step;
	int	m_Count;
	Boolean m_fClear;
	int 	m_elementMax;
	IPAIR *	m_array;
	static unsigned int s_totalMem;

	/** Reallocate to newmax */
	Boolean setmax(int newmax);
	/** Grow to allow n items in array */
	Boolean grow(int n);

	public:
	MIPairList(int start=4);
	~MIPairList();
	int	FindKey(MPLKEY key, int *pndxret=NULL);
	int	count(void);
	void 	trunk(int ndx);
	Boolean	AddIndex(MPLKEY key, MPLVAL value, int ndx);
	MPLVAL & operator [] (MPLKEY);

	inline MPLKEY GetKey(int ndx)
	{	return (ndx >= 0 && ndx < m_Count) ? m_array[ndx].key : 
			(ndx > m_Count) ? m_array[m_Count-1].key : m_array[0].key;
	}
	inline MPLVAL GetValue(int ndx)
	{	return (ndx >= 0 && ndx < m_Count) ? m_array[ndx].value : 
			(ndx > m_Count) ? m_array[m_Count-1].value : m_array[0].value;
	}
	static inline unsigned int GetTotalMem()
	{
		return s_totalMem;
	}
};

template <class TYPE> class MTSparseArray
{
	protected:
	MIPairList	m_mip;

	public:

	MTSparseArray()
	{
	}
	~MTSparseArray()
	{
		DeleteAll();
	}
	void DeleteAll(void)
	{
		int count = m_mip.count();
		for(int i=0; i < count; i++)
		{
			TYPE *p = (TYPE *) m_mip.GetValue(i);
			delete p;
		}
	}
	TYPE & operator [] (MPLKEY key)
	{
		int ndx;
		int ndxinsert;
		TYPE  *pt = NULL;

		ndx = m_mip.FindKey(key, &ndxinsert);

		if(ndx == -1)	// New value
		{
			pt =  new TYPE ();
			m_mip.AddIndex(key, (MPLVAL)pt, ndxinsert);
			ndx = ndxinsert;
		}
		else
		{
			pt = (TYPE *) m_mip[ndx];
		}
		return (*pt);
	}
	int count(void)
	{
		return m_mip.count(); 
	}
	MIPairList * Get(int ndx)
	{
		return (MIPairList *) m_mip.GetValue(ndx);
	}
};
#define CONTAINER_SPARSE_ARRAY( CNAME )					\
template <class TYPE> class MT##CNAME##SparseArray 			\
{									\
	protected:							\
	CNAME	m_cnt;							\
	class MIntNodeT : public MIntNode				\
	{ 	public:							\
		MIntNodeT(int key, NData data) : MIntNode(key,data) { } \
		~MIntNodeT() { delete (TYPE *) m_data; }		\
	};								\
	public:								\
	MT##CNAME##SparseArray() { }					\
	~MT##CNAME##SparseArray() { }					\
	TYPE & operator [] (MPLKEY key)					\
	{	TYPE  *pt = NULL;					\
		MNode *mn = m_cnt.Find((NKey)key);			\
		if(!mn)							\
		{ 	pt = new TYPE ();				\
			mn = new MIntNodeT((int)key, (NData)pt);	\
			m_cnt.Insert(mn);				\
		}							\
		else							\
			pt = ( TYPE * )mn->DataGet();			\
		return (*pt);						\
	}								\
	void DeleteAll(void) { m_cnt.DeleteAll(); }			\
}

CONTAINER_SPARSE_ARRAY ( MHash );

CONTAINER_SPARSE_ARRAY ( MAVL );


#endif
