/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MSERVER_
#define _MSERVER_

/** A threaded TCP/IP server */
class MServerThreadCore : public MThread
{

	protected:
	struct tms	m_tmsStart;
	struct tms	m_tmsEnd;

	clock_t		m_ctStart;
	clock_t		m_ctEnd;


	public:
	MServerThreadCore();
	~MServerThreadCore();

	inline clock_t	Ticks(int mult=1)
	{
#ifdef WIN32
		return GetTickCount();
#else
		struct tms tmsT;
		clock_t t = times(&tmsT);
		return mult*(t - m_ctStart);
#endif
	}
	
	virtual Boolean PreAccept(void);
	virtual void PrepSocket(MSock *ms);
	virtual Boolean HandleSocket(MSock *ms)=0;
	virtual void CloseSocket(MSock *ms);
	virtual void CleanUp(void);
};

class MServerThread : public MServerThreadCore
{
	protected:
	MSock *		m_listener;
	void threadLoop(void);

	public:
	MServerThread(MSock *listener);
	~MServerThread();
};


class MThreadedServer : public MThreadMgr
{
	protected:
	

	virtual	Boolean listen_wait(void);
	public:
	void Run(MSock *listener, int queue);
};

class MServerThThread : public MServerThreadCore
{
	protected:
	MMutex	m_mtx;
	MSock	m_sock;
	void threadLoop(void);
	
	public:

	MServerThThread();
	~MServerThThread();
	Boolean Accept(MSock *ms);
};



#endif
