/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MTSORT_H_
#define _MTSORT_H_

#ifndef COMPARE_DEFINED
#define COMPARE_DEFINED
typedef int (*COMPARE)(const void *a, const void *b);
#endif

/** a template class for sorting data */
template <class TYPE> class MTSort 
{
		
	protected:
	inline void swap(TYPE &a, TYPE &b)
	{
		TYPE tmp;
		tmp = a;
		a = b;
		b = tmp;
	}
	inline Boolean tcomp(TYPE &a, TYPE &b)
	{
		return (a - b) > 0;
	}
	public:
	/** 
		three element sort, 2.5 avg compares.
		Moves data more than once. 
	*/
	void three(TYPE *array, COMPARE comp)	
	{			
		if(comp((PVOID)&array[0],(PVOID)&array[2])>0)	
			swap(array[0],array[2]);
		if(comp((PVOID)&array[1],(PVOID)&array[2])>0)
			swap(array[1], array[2]);
		else if(comp((PVOID)&array[0],(PVOID)&array[1])>0)
			swap(array[0], array[1]);
	}
	/**
	 	Three element sort with built in compare.
	*/
	void three(TYPE *array)	
	{			
		if(tcomp(array[0],array[2]))	
			swap(array[0],array[2]);
		if(tcomp(array[1],array[2]))
			swap(array[1],array[2]);
		else if(tcomp(array[0],array[1]))
			swap(array[0], array[1]);
	}


	/** 
		four element sort, 5 compares.
		Moves data more than once.
	*/
	void four(TYPE *array, COMPARE comp)
	{		
		if(comp(&array[0],&array[3])>0)
			swap(array[0],array[3]);
		if(comp(&array[1],&array[2])>0)
			swap(array[1],array[2]);
		if(comp(&array[2],&array[3])>0)
			swap(array[2],array[3]);
		if(comp(&array[0],&array[1])>0)
			swap(array[0],array[1]);
		if(comp(&array[1],&array[2])>0)
			swap(array[1],array[2]);
	}
	/**
	 	four element sort with builty in compare
	 */
	void four(TYPE *array)
	{		
		if(tcomp(array[0],array[3]))
			swap(array[0],array[3]);
		if(tcomp(array[1],array[2]))
			swap(array[1],array[2]);
		if(tcomp(array[2],array[3]))
			swap(array[2],array[3]);
		if(tcomp(array[0],array[1]))
			swap(array[0],array[1]);
		if(tcomp(array[1],array[2]))
			swap(array[1],array[2]);
	}

	/**
		Shell sort.
		Moves data more than once.
	*/
	void shell(TYPE *array, int n, COMPARE comp)
	{
		int  i, j;
		TYPE t;	
		int h = 1;
		if (n >= 14)
		{		
			while (h < n)
				h = 3*h + 1;
			h /=9;	
		}	
		while (h > 0) 			
		{
			for (i = h; i < n; i++) 
			{
				t = array[i];
				for (j = i-h; j >= 0 && comp((PVOID)&array[j],(PVOID)&t)>0; j -= h)
					array[j+h] = array[j];
				array[j+h] = t;
			}
			h /= 3;	
		}
	}
	/**
	 	shell sort with built-in compare function.
	*/
	void shell(TYPE *array, int n)
	{
		int  i, j;
		TYPE t;	
		int h = 1;
		if (n >= 14)
		{		
			while (h < n)
				h = 3*h + 1;
			h /=9;	
		}	
		while (h > 0) 			
		{
			for (i = h; i < n; i++) 
			{
				t = array[i];
				for (j = i-h; j >= 0 && tcomp(array[j],t); j -= h)
					array[j+h] = array[j];
				array[j+h] = t;
			}
			h /= 3;	
		}
	}
};

#endif
