/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MUTILS_
#define _MUTILS_

#include <ctype.h>
#include <math.h>
#ifdef Linux
#include <sys/times.h>
#endif

#include "metadef.h"

#define INLINE_STROPS	TRUE

extern char g_toupper_tbl[];
extern char g_tolower_tbl[];
extern char g_isalnum_tbl[];
extern char g_isvowel_tbl[];

#ifdef NEVER
#define ISVOWEL(C)	(C=='a' || C=='e' || C=='i' || C=='o' || C=='u')
#endif

#ifdef WIN32
struct tms 
{
	clock_t tms_utime;  /* user time */
	clock_t tms_stime;  /* system time */
	clock_t tms_cutime; /* user time of children */
	clock_t tms_cstime; /* system time of children */
};
extern clock_t times(struct tms *t);
#endif

#define ISALNUM(c) 	(g_isalnum_tbl[((unsigned char)(c))])
// #define ISVOWEL(c)	(g_isvowel_tbl[((unsigned char)(c))])
#define UCP(p)		((unsigned char *)(p))

class MFile;

typedef Boolean (*TOKENFN)(char *token, void *param1, void *param2);
// in mutil.cpp

#ifdef INLINE_STROPS
#define STRDECL	inline
#include "mstrops.h"
#else
char *trim(char *s);
char *strrev(char *p);
char *stripnl(char *s);
char *stripnl2lo(char *str);
char *stripnl2lo(char *dest, char *src);
int match(char *str, char *pat);
char *strcasestr(char *haystack, char *needle);
#endif

char *snprintf(char *buffer, size_t cb, char *fmt, ...);
int ncasematch(char *str, char *pat);
void trace_exit(char *file, int line, char *msg);
unsigned int hextoi(char *s);

// In mitoa.cpp
char *_itoa(int val, char *sz, int radix);
char *_ltoa(int val, char *sz, int radix);
char *_ultoa(int val, char *sz, int radix);

unsigned long GetMilliTicks(void);
unsigned long GetDiffTicks(unsigned long start_ticks);

int TokenMetaphone(char *dest, char *str);
int VowelMetaphone(char *dest, char *str);
int BasicMetaphone(char *dest, char *str);

// Modec handling routines.
char *itomodec(int val, char *p, Boolean is_signed=0);
unsigned int modectoi(char *s);
char modec(int n);

void RandString(char *buffer, int size);
double CashRound(double number);

/** A class to wrap malloc calls. Allows deletion during cleanup phase of
exceptions */

class AutoMalloc
{
protected:
        void    *m_pointer;

public:
        AutoMalloc(size_t size);
        virtual ~AutoMalloc();
        virtual char *ch_ptr(void);
};
      
class MRandomFile ;

#define CMPEXTRA	16

void *Compress(void *m, int *pcb, void *mem);
void *Decompress(void *pcmp, int *pcb, void *pdest);
void *Decompress(MRandomFile *mf, int *pcb, void *pdest);

class DynamicBuffer
{
	protected:
	int	m_cb;
	int	m_jump;
	void *	m_data;
	public:
	DynamicBuffer(int cb = 256);
	~DynamicBuffer();
	void *GetPointer(int len);
	void *GetZPointer(int len);
	void *GetPointer(int n, int cb);
	char *Strcpy(char *str);
	char *ptr(void)	{ return (char *) m_data; }

#ifdef POSTGRESQL_INC
	char *GetPointer(text *text);
	char *SizePointer(text *text);
#endif
};

extern void InitPhoenix(int argc, char **argv);

#define ATOMIC_INC(int_num) atom_inc((volatile int *)&(int_num))
#define ATOMIC_DEC(int_num) atom_dec((volatile int *)&(int_num))
#define ATOMIC_XCHNG(num,val) atom_xchg((volatile int *)&(num), val)

#ifdef GCC
#define ATOMIC_OPS
static __inline__ void atom_inc(volatile int *num)
{
        __asm__ __volatile__ ( "lock incl %0" : "=m" (*num));
}

static __inline__ void atom_dec(volatile int *num)
{
        __asm__ __volatile__ ( "lock decl %0" : "=m" (*num));
}

static __inline__ int atom_xchg(volatile int *m, int inval)
{
	register int val = inval;
	__asm__ __volatile__ ( "xchg %1,%0" : "=m" (*m), "=r" (val) : "1" (inval));
        return val;
}

#endif

/** A mutex controlled counter, uses optimized ATOMIC_OPS if possible */
class AtomicCounter
{
	int	m_count;
#ifndef ATOMIC_OPS
	MMutex	mtx;
#endif
	public:
	AtomicCounter(void)
	{
		m_count=0;
	}
	void Inc(void )
	{
#ifndef ATOMIC_OPS 
		mtx.lock();
		m_count++;
		mtx.unlock();
#else
		ATOMIC_INC(m_count);
#endif
	}
	void Dec(void )
	{
#ifndef ATOMIC_OPS 
		mtx.lock();
		m_count--;
		mtx.unlock();
#else
		ATOMIC_DEC(m_count);
#endif
	}
	int Get()
	{ return m_count; }
	void Set(int n)
	{ m_count = n; }
	
};


/** A auto counter, increments an AtomicCounter on creation, and 
decrements that counter on deletion */
class AutoCounter
{
	protected:
	AtomicCounter *	m_ac;
	public:
	AutoCounter(AtomicCounter *ac) { m_ac = ac; ac->Inc(); }
	~AutoCounter() { m_ac->Dec(); }
};


// Main server thread class.

#ifdef Win32
void Win32_printerr(char *title, Boolean fPopup);

#endif
#endif
